@@ -303,24 +303,28 @@ static int check_volser(char *s, int devno)
 /*
  * do some blocksize checks
  */
-static int check_param(char *s, format_data_t *data)
+static int check_param(char *s, size_t buffsize, format_data_t *data)
 {
 	int tmp = data->blksize;
 
 	if ((tmp < 512) || (tmp > 4096)) {
-		strcpy(s,"Blocksize must be one of the following positive "
-		       "integers:\n512, 1024, 2048, 4096.");
+		strncpy(s,"Blocksize must be one of the following positive "
+			"integers:\n512, 1024, 2048, 4096.", buffsize);
+		if (buffsize > 0)
+			s[buffsize - 1] = '\0';
 		return -1;
 	}
 
 	while (tmp > 0) {
 		if ((tmp % 2) && (tmp != 1)) {
-			strcpy(s,"Blocksize must be a power of 2.");
+			strncpy(s,"Blocksize must be a power of 2.", buffsize);
+			if (buffsize > 0)
+				s[buffsize - 1] = '\0';
 			return -1;
 		}
 		tmp /= 2;
 	}
-	
+
 	return 0;
 }
 
@@ -350,7 +354,7 @@ static format_data_t ask_user_for_blksize(format_data_t params)
 		if (rc != 1) 
 			printf(" -- wrong input, try again.\n");
 
-		if (check_param(str, &params) < 0) {
+		if (check_param(str, ERR_LENGTH, &params) < 0) {
 			printf(" -- %s\n",str); 
 			rc = 0; 
 		}
@@ -1043,7 +1047,7 @@ int main(int argc,char *argv[])
 
 	check_disk(&info);
 
-	if (check_param(str, &format_params) < 0)
+	if (check_param(str, ERR_LENGTH, &format_params) < 0)
 		ERRMSG_EXIT(EXIT_MISUSE, "%s: %s\n", prog_name, str);
 
 	do_format_dasd(&info, &format_params, &vlabel);
