@@ -3,7 +3,7 @@
  * Author(s)......: Volker Sameske <sameske@de.ibm.com>
  *                  Horst Hummel   <horst.hummel@de.ibm.com>
  *                  Gerhard Tonn   <ton@de.ibm.com>
- * Copyright IBM Corp. 2001,2007
+ * Copyright IBM Corp. 2001,2011
  */
 
 #include <getopt.h>
@@ -21,7 +21,7 @@ char *line_ptr = line_buffer;
 static const char tool_name[] = "fdasd: zSeries DASD partitioning program";
 
 /* Copyright notice */
-static const char copyright_notice[] = "Copyright IBM Corp. 2001, 2006";
+static const char copyright_notice[] = "Copyright IBM Corp. 2001, 2011";
 
 /*
  * Print version information.
@@ -145,6 +145,7 @@ fdasd_cleanup (fdasd_anchor_t *anchor)
 	part_info = anchor->first;
 	for (i = 1; i <= USABLE_PARTITIONS && part_info != NULL; i++) {
 		next = part_info->next;
+		free(part_info->f1);
 		free(part_info);
 		part_info = next;
 	}
@@ -312,6 +313,10 @@ fdasd_partition_type (char *str)
 		strcpy(str, "Linux native");
 	else if (strncmp("SWAP  ", str, 6) == 0)
 		strcpy(str, "Linux swap");
+	else if (strncmp("RAID  ", str, 6) == 0)
+		strcpy(str, "Linux raid");
+	else if (strncmp("LVM   ", str, 6) == 0)
+		strcpy(str, "Linux lvm");
 	else
 		strcpy(str, "unknown");
 
@@ -380,7 +385,7 @@ fdasd_menu (void)
 static void
 fdasd_initialize_anchor (fdasd_anchor_t *anc) 
 {
-	partition_info_t *part_info, *prev_part_info;
+	partition_info_t *part_info, *prev_part_info = NULL;
 	volume_label_t *vlabel;
 	int i;
 
@@ -424,29 +429,18 @@ fdasd_initialize_anchor (fdasd_anchor_t *anc)
 	anc->vlabel = vlabel;
 
 	for (i=1; i<=USABLE_PARTITIONS; i++) {
-	        part_info = malloc(sizeof(partition_info_t));
+		part_info = malloc(sizeof(partition_info_t));
 		if (part_info == NULL) 
 			fdasd_error(anc, malloc_failed,
 				   "Partition info memory allocation failed.");
-		part_info->used       = 0x00;
-		part_info->len_trk    = 0;
-		part_info->start_trk  = 0;
-		part_info->fspace_trk = 0;
+		memset(part_info, 0, sizeof(partition_info_t));
 
 		/* add part_info to double pointered list */
 		if (i == 1) {
-		        anc->first = part_info;
-			part_info->prev = NULL;
-		} 
-		else if (i == USABLE_PARTITIONS) {
-		        anc->last = part_info;
-		        part_info->next = NULL;
-			part_info->prev = prev_part_info;
-			prev_part_info->next = part_info;
-		} 
-		else {
-		        part_info->prev = prev_part_info;
-		        prev_part_info->next = part_info;
+			anc->first = part_info;
+		} else if (i == USABLE_PARTITIONS) {
+			anc->last = part_info;
+			part_info->next = NULL;
 		}
 
 		part_info->f1 = malloc(sizeof(format1_label_t));
@@ -454,7 +448,13 @@ fdasd_initialize_anchor (fdasd_anchor_t *anc)
 			fdasd_error(anc, malloc_failed,
 				    "FMT1 DSCB memory allocation failed.");
 		bzero(part_info->f1, sizeof(format1_label_t));
-		
+
+		if (prev_part_info) {
+			prev_part_info->next = part_info;
+			part_info->prev = prev_part_info;
+		} else {
+			part_info->prev = NULL;
+		}
 		prev_part_info = part_info;
 	}
 	anc->hw_cylinders = 0;
@@ -534,6 +534,21 @@ fdasd_parse_options (fdasd_anchor_t *anc, struct fdasd_options *options,
 	options->device = argv[optind]; 	
 }
 
+int gettoken(char *str, char *ch, char *token[], int max)
+{
+	int i;
+
+	token[0] = strtok(str, ch);
+	if (!token[0])
+		return 0;
+
+	for (i = 1; i < max; i++) {
+		token[i] = strtok(NULL, ch);
+		if (!token[i])
+			break;
+	}
+	return i;
+}
 
 /*
  * parses config file
@@ -541,10 +556,10 @@ fdasd_parse_options (fdasd_anchor_t *anc, struct fdasd_options *options,
 static int
 fdasd_parse_conffile(fdasd_anchor_t *anc, struct fdasd_options *options) 
 {
-	char buffer[USABLE_PARTITIONS * LINE_LENGTH];
-	char err_str[ERROR_STRING_SIZE], *c1, *c2;
+	char buffer[CONFIG_FILE_SIZE + 1];
+	char err_str[ERROR_STRING_SIZE], *c1, *c2, *token[CONFIG_MAX];
 	int fd, rc;
-	unsigned i=0;
+	int i;
 
 	/* if name of config file was not specified, select the default */
 	if (options->conffile == NULL)
@@ -560,21 +575,19 @@ fdasd_parse_conffile(fdasd_anchor_t *anc, struct fdasd_options *options)
 		fdasd_error(anc, unable_to_open_disk, err_str);
 	}
 
-	bzero(buffer, sizeof(buffer));
-	rc = read(fd, buffer, sizeof(buffer));
+	memset(buffer, 0, sizeof(buffer));
+	rc = read(fd, buffer, sizeof(buffer) - 1);
 	if (rc < 0)
 		return -1;
 	close(fd);
 
 
-	for (i=0; i<sizeof(buffer); i++) 
+	for (i = 0; i < rc; i++)
 		buffer[i] = toupper(buffer[i]);
 
 	c1 = buffer;
 
 	for (i=0; i<USABLE_PARTITIONS; i++) {
-		char *token, *stopstring;
-
 		c1 = strchr(c1, '[');
 		if (c1 == NULL) {
 			if (!anc->silent)
@@ -593,18 +606,57 @@ fdasd_parse_conffile(fdasd_anchor_t *anc, struct fdasd_options *options)
 		}
 		strcpy(c2, "");
 
-		token = strtok(c1, ",");
-		if (strstr(token, "FIRST") != NULL)
-			anc->confdata[i].start=FIRST_USABLE_TRK;
-		else
-			anc->confdata[i].start=strtol(token, &stopstring, 10);
+		memset(token, 0, sizeof(token));
+		if (gettoken(c1, ",", token, CONFIG_MAX) < 2) {
+			snprintf(err_str, ERROR_STRING_SIZE,
+				 "Missing parameter in config file "	\
+				 "%s\n", options->conffile);
+			fdasd_error(anc, config_syntax_error, err_str);
+		}
 
-		token = strtok(NULL, ",");
-		if (strstr(token, "LAST") != NULL)
+		if (strstr(token[0], "FIRST") != NULL)
+			anc->confdata[i].start = FIRST_USABLE_TRK;
+		else {
+			errno = 0;
+			anc->confdata[i].start = strtol(token[0],
+							(char **) NULL, 10);
+			if (errno != 0 || anc->confdata[i].start == 0) {
+				snprintf(err_str, ERROR_STRING_SIZE,
+					 "invalid partition start in config"
+					 "file %s\n", options->conffile);
+				fdasd_error(anc, config_syntax_error, err_str);
+			}
+		}
+
+		if (strstr(token[1], "LAST") != NULL)
 			anc->confdata[i].stop = anc->formatted_cylinders
 				* geo.heads - 1;
-		else
-			anc->confdata[i].stop = strtol(token, &stopstring, 10);
+		else {
+			errno = 0;
+			anc->confdata[i].stop = strtol(token[1],
+						       (char **) NULL, 10);
+			if (errno != 0 || anc->confdata[i].stop == 0) {
+				snprintf(err_str, ERROR_STRING_SIZE,
+					 "invalid partition end in config"
+					 "file %s\n", options->conffile);
+				fdasd_error(anc, config_syntax_error, err_str);
+			}
+		}
+
+		if (token[2] == NULL || strstr(token[2], "NATIVE") != NULL)
+			anc->confdata[i].type = PARTITION_NATIVE;
+		else if (strstr(token[2], "SWAP") != NULL)
+			anc->confdata[i].type = PARTITION_SWAP;
+		else if (strstr(token[2], "RAID") != NULL)
+			anc->confdata[i].type = PARTITION_RAID;
+		else if (strstr(token[2], "LVM") != NULL)
+			anc->confdata[i].type = PARTITION_LVM;
+		else {
+			snprintf(err_str, ERROR_STRING_SIZE,
+				 "invalid partition type in config file %s\n",
+				 options->conffile);
+			fdasd_error(anc, config_syntax_error, err_str);
+		}
 		c1 = c2 + 1;
 	}
 
@@ -682,6 +734,7 @@ fdasd_check_conffile_input (fdasd_anchor_t *anc,
 		part_info->start_trk = start;
 		part_info->end_trk   = stop;
 		part_info->len_trk   = stop - start + 1;
+		part_info->type      = anc->confdata[i].type;
 
 		/* update the current free space counter */
 		if (i == 0)
@@ -1122,7 +1175,14 @@ fdasd_write_vtoc_labels (fdasd_anchor_t *anc)
 			strncpy(c1, s2, 31);
 		}
 		else {
-			char *tmp = strstr(ch, "SWAP");
+			if (strstr(ch, "SWAP") != NULL)
+				part_info->type = PARTITION_SWAP;
+			else if (strstr(ch, "RAID") != NULL)
+				part_info->type = PARTITION_RAID;
+			else if (strstr(ch, "LVM") != NULL)
+				part_info->type = PARTITION_LVM;
+			else
+				part_info->type = PARTITION_NATIVE;
 
 			/* create a new data set name */
 			while (getpos(anc, k) > -1)
@@ -1145,10 +1205,21 @@ fdasd_write_vtoc_labels (fdasd_anchor_t *anc)
 			strncpy(c1, dsno, 4);
 
 			c1 += 4;
-			if (tmp)
-				strncpy(c1, ".SWAP", 5);
-			else
+
+			switch (part_info->type) {
+			case PARTITION_NATIVE:
 				strncpy(c1, ".NATIVE", 7);
+				break;
+			case PARTITION_SWAP:
+				strncpy(c1, ".SWAP", 5);
+				break;
+			case PARTITION_RAID:
+				strncpy(c1, ".RAID", 5);
+				break;
+			case PARTITION_LVM:
+				strncpy(c1, ".LVM", 4);
+				break;
+			}
 		}
 		vtoc_ebcdic_enc(ch, ch, 44);
 		if (anc->verbose) printf("%2x ", part_info->f1->DS1FMTID);
@@ -1433,22 +1504,31 @@ fdasd_change_part_type (fdasd_anchor_t *anc)
 
 	printf("current partition type is: %s\n\n", fdasd_partition_type(str));
 	printf("   1  Linux native\n" \
-	       "   2  Linux swap\n\n");
+	       "   2  Linux swap\n" \
+	       "   3  Linux raid\n" \
+	       "   4  Linux lvm\n\n");
 	part_type = 0;
-	while ((part_type < 1) || (part_type > 2)) {
-        	while (!isdigit(part_type = 
+	while ((part_type < 1) || (part_type > 4)) {
+        	while (!isdigit(part_type =
 				read_char("new partition type: ")));
         	part_type -= 48;
 	}
 
         switch (part_type) {
-        case 1:
+	case PARTITION_NATIVE:
 		strncpy(str, "NATIVE", 6);
-                break;
-        case 2:
+		break;
+	case PARTITION_SWAP:
 		strncpy(str, "SWAP  ", 6);
-                break;
-        default:
+		break;
+	case PARTITION_RAID:
+		strncpy(str, "RAID  ", 6);
+		break;
+	case PARTITION_LVM:
+		strncpy(str, "LVM   ", 6);
+		break;
+
+	default:
                 printf("'%d' is not supported!\n", part_type);
         }
 
@@ -1640,6 +1720,7 @@ fdasd_process_valid_vtoc(fdasd_anchor_t *anc, unsigned long blk)
 		anc->formatted_cylinders = anc->f4->DS4DEVCT.DS4DSCYL;
 	anc->fspace_trk = anc->formatted_cylinders * geo.heads
 		- FIRST_USABLE_TRK;
+	/* skip f4 label, already read before */
 	blk += anc->blksize;
 
 	if (anc->formatted_cylinders < anc->hw_cylinders)
@@ -1649,7 +1730,8 @@ fdasd_process_valid_vtoc(fdasd_anchor_t *anc, unsigned long blk)
 
 	if (anc->verbose) printf("VTOC DSCBs          : ");
 
-	for (i = 1; i <= geo.sectors; i++) {
+	/* go through remaining labels, f4 label already done */
+	for (i = 1; i < geo.sectors; i++) {
 		bzero(&f1_label, f1_size);
 		vtoc_read_label(options.device, blk, &f1_label, NULL, NULL,
 				NULL);
@@ -1806,6 +1888,7 @@ fdasd_check_volume (fdasd_anchor_t *anc)
 	long long blk = -1;
 	char str[LINE_LENGTH];
 	char inp_buf[INPUT_BUF_SIZE];
+	int rc = 1;
 
 	if (!anc->silent)
 		printf("reading volume label ..:");
@@ -1835,8 +1918,9 @@ fdasd_check_volume (fdasd_anchor_t *anc)
 	} else {
 	        /* didn't find VOL1 volume label */
 
-		if (anc->print_table) {
-			printf("\nCould not find VOL1 volume label.\n");
+ 		if (anc->print_table || anc->print_volser) {
+			printf("\nCannot show requested information because "
+			       "the disk label block is invalid\n");
 			return -1;
 		}
 
@@ -1849,12 +1933,12 @@ fdasd_check_volume (fdasd_anchor_t *anc)
 		else {
 			if (!anc->silent)
 				printf(" no known label\n");
-			strcpy(inp_buf,"Should I create a new one?");
+			if (!anc->auto_partition)
+				rc = yes_no("Should I create a new one?");
 		}
-                if ((!anc->print_volser) && (!anc->print_table) && 
-		    (yes_no(inp_buf) == 1)) {
-		        printf("You need a VOL1 volume label for " \
-		               "partitioning\nexiting...\n");
+                if ((!anc->print_volser) && (!anc->print_table) && (rc == 1)) {
+			printf("Disc does not contain a VOL1 label, cannot "
+			       "create partitions.\nexiting... \n");
 			fdasd_exit(anc, -1);
                 }
 
@@ -1906,7 +1990,7 @@ fdasd_check_disk_access (fdasd_anchor_t *anc)
 	format1_label_t f1;
 	int fd, pos, ro;
 
-        if ((fd = open(options.device, O_RDWR)) == -1) {
+        if ((fd = open(options.device, O_RDONLY)) == -1) {
 		snprintf(err_str, ERROR_STRING_SIZE,
 			"Could not open device '%s' " \
 			"in read-only mode!\n", options.device);
@@ -1961,7 +2045,7 @@ fdasd_get_geometry (fdasd_anchor_t *anc)
         int fd, blksize = 0;
 	dasd_information_t dasd_info;
 	char err_str[ERROR_STRING_SIZE];
-	u_int64_t device_size;
+	struct dasd_eckd_characteristics *characteristics;
 
 	if ((fd = open(options.device,O_RDONLY)) < 0) {
 		snprintf(err_str, ERROR_STRING_SIZE,
@@ -1982,14 +2066,6 @@ fdasd_get_geometry (fdasd_anchor_t *anc)
 			    "Could not retrieve blocksize information.");
 	}
 
-	if (ioctl(fd, BLKGETSIZE64, &device_size) != 0) {
-		close(fd);
-		fdasd_error(anc, unable_to_ioctl,
-			    "Could not retrieve device size information.");
-	}
-
-	anc->hw_cylinders = ((device_size / blksize) / geo.sectors) / geo.heads;
-
 	/* get disk type */
 	if (ioctl(fd, BIODASDINFO, &dasd_info) != 0) {
 	        close(fd);
@@ -1997,6 +2073,14 @@ fdasd_get_geometry (fdasd_anchor_t *anc)
 			    "Could not retrieve disk information.");
 	}
 
+	characteristics =
+		(struct dasd_eckd_characteristics *) &dasd_info.characteristics;
+	if (characteristics->no_cyl == LV_COMPAT_CYL &&
+	    characteristics->long_no_cyl)
+		anc->hw_cylinders = characteristics->long_no_cyl;
+	else
+		anc->hw_cylinders = characteristics->no_cyl;
+
 	close(fd);
 
 	if (strncmp(dasd_info.type, "ECKD", 4) != 0) {
@@ -2581,7 +2665,8 @@ fdasd_auto_partition_conffile(fdasd_anchor_t *anc)
 	unsigned long start, stop;
 	extent_t ext;
 	cchhb_t hf1;
-	char old_volser[VOLSER_LENGTH + 1];
+	char old_volser[VOLSER_LENGTH + 1], *type;
+
 
 	vtoc_volume_label_init(vlabel);
 	vtoc_volume_label_set_key(vlabel, "VOL1");
@@ -2660,6 +2745,21 @@ fdasd_auto_partition_conffile(fdasd_anchor_t *anc)
 					   anc->verbose, start, stop,
 					   anc->formatted_cylinders, geo.heads);
 		}
+
+		/* write correct partition type */
+		vtoc_ebcdic_dec(part_info->f1->DS1DSNAM,
+				part_info->f1->DS1DSNAM, 44);
+		type = strstr(part_info->f1->DS1DSNAM, ".NEW");
+		if (part_info->type == PARTITION_SWAP)
+			strncpy(type, ".SWAP", 5);
+		else if (part_info->type == PARTITION_RAID)
+			strncpy(type, ".RAID", 5);
+		else if (part_info->type == PARTITION_LVM)
+			strncpy(type, ".LVM", 4);
+		else
+			strncpy(type, ".NATIVE", 7);
+		vtoc_ebcdic_enc(part_info->f1->DS1DSNAM,
+				part_info->f1->DS1DSNAM, 44);
 	} while ((part_info = part_info->next) != NULL);
 
 	anc->vlabel_changed++;
@@ -2728,14 +2828,14 @@ main(int argc, char *argv[])
 		fdasd_auto_partition_conffile(&anchor);
 	}
 
-	if (anchor.print_table) {
-		if (rc == 0)
-			fdasd_list_partition_table(&anchor);
+	if (anchor.print_volser) {
+		fdasd_print_volser(&anchor);
 		fdasd_quit(&anchor);
 	}
 
-	if (anchor.print_volser) {
-		fdasd_print_volser(&anchor);
+	if (anchor.print_table) {
+		if (rc == 0)
+			fdasd_list_partition_table(&anchor);
 		fdasd_quit(&anchor);
 	}
 
