@@ -22,6 +22,7 @@
 
 #define DEV_BUFFER_LENGTH 20
 #define PATH_BUFFER_LENGTH 256
+#define BUSIDSIZE 8
 
 #define BLOCKPATH "/sys/block/"
 #define DEVICE_LINK "device"
@@ -212,6 +213,42 @@ static int find_busid_in_sysfs(char *dev, char *busid) {
 		return -1;
 }
 
+/*
+ * helper function that searches for a specific block device in
+ * /proc/dasd/devices and returns it's bus-ID
+ * maja, mina: <major>, <minor> of the device
+ * busid: buffer in which the bus-ID string will be returned
+ * returns 0 for successful operation and -1 in case of an error
+ * e.g. /proc/dasd/devices does not exist.
+ *
+ * An entry looks like:
+ * 0.0.XXXX(DISCIPLINE) at ( MAJ:   MIN) is dasdX       :
+ * active at blocksize: BLOCKSIZE, BLOCKS blocks, SIZE MB
+ */
+static int find_busid_in_proc(int maja, int mina, char *busid)
+{
+	FILE *filp;
+	char bus[BUSIDSIZE + 1];
+	int majb, minb, rc;
+
+	rc = -1;
+
+	filp = fopen("/proc/dasd/devices", "r");
+	if (!filp)
+		return rc;
+	while (fscanf(filp, "%[^(] %*s at ( %d : %d %*[^\n]\n",
+		      bus, &majb, &minb) != EOF) {
+		if ((maja == majb) && (mina == minb)) {
+			strncpy(busid, bus, BUSIDSIZE);
+			rc = 0;
+			break;
+		}
+	}
+
+	fclose(filp);
+	return rc;
+}
+
 /*
  * Return the busid of a given device node.
  * Works only for block devices.
@@ -236,9 +273,12 @@ int u2s_getbusid(char *devicenode, char *busid)
                 return -1;
         maj = major(stat_buf.st_rdev);
         min = minor(stat_buf.st_rdev);
-        snprintf(dev_string,DEV_BUFFER_LENGTH,"%u:%u", maj, min);
 
-	rc = find_busid_in_sysfs(dev_string, busid);
+	rc = find_busid_in_proc(maj, min, busid);
+	if (rc) {
+		snprintf(dev_string, DEV_BUFFER_LENGTH, "%u:%u", maj, min);
+		rc = find_busid_in_sysfs(dev_string, busid);
+	}
 
 	return rc;
 }
