@@ -627,7 +627,7 @@ int register_tables ( void* mib_data,
   void        *mib_data_ptr;      /* ptr to parse IPAssists MIB info */
 
   TABLE_OID      *oid_ptr, *ins_oid = NULL; /* ptr into Toplevel OID linked list */
-  REG_INDICES    *ind_ptr, *ins_ind; /* ptr into index linked list */
+  REG_INDICES    *ind_ptr;        /* ptr into index linked list */
  
   /* ptr to ucd-snmp variable13 structs that will contain the suffix
      information registered with subagent driving code */
@@ -862,9 +862,9 @@ int register_tables ( void* mib_data,
 	  } /* end if */
 
 	  strcpy( new_index, (char*) mib_data_ptr );
-	  ins_ind = index_insert_after( new_index,
-					mib_data_hdr->ioctl_cmd.ipa_cmd_hdr.ifIndex,
-					ind_ptr );
+	  index_insert_after( new_index,
+				mib_data_hdr->ioctl_cmd.ipa_cmd_hdr.ifIndex,
+				ind_ptr );
 	} /* end if */	  
 
       } /* end for (j) */
@@ -996,7 +996,7 @@ int header_osa_table( struct variable *vp, oid *name, size_t *length,
 	index_len = (int) (*length) - (int) vp->namelen;
       
       DEBUGMSG(("ibmOSAMib-Subagent", 
-		  " header_osa_table - GET index length: %d\n", index_len));
+		  " header_osa_table - GET index length: %zd\n", index_len));
  
       /* search our internal linked list for a matching Toplevel OID */
       res = search_top_oid ( (oid*) vp->name, (size_t) vp->namelen,
