@@ -645,8 +645,12 @@ overwrite_partition_start(int fd, struct disk_info* info, int mv_dump_magic)
 	return 0;
 }
 
-
-static int check_partition_bounds(struct disk_info* info)
+/*
+ * Ensure that end block is within bounds.
+ * Force block size of 4KiB because otherwise there is not enough space
+ * to write the dump tool.
+ */
+static int check_eckd_dump_partition(struct disk_info* info)
 {
 	unsigned long long end_blk = info->geo.start + info->phy_blocks - 1;
 
@@ -658,6 +662,11 @@ static int check_partition_bounds(struct disk_info* info)
 				info->phy_block_size) >> 20);
 		return -1;
 	}
+	if (info->phy_block_size != 4096) {
+		error_reason("unsupported DASD block size %d (should be 4096)",
+			     info->phy_block_size);
+		return -1;
+	}
 	return 0;
 }
 
@@ -668,7 +677,7 @@ struct eckd_dump_param {
 	uint8_t		num_heads;
 	uint8_t		bpt;
 	char		reserved[4];
-} __attribute((packed));
+} __attribute((packed, may_alias));
 
 static void eckd_dump_store_param(struct eckd_dump_param* param,
 				  struct disk_info* info)
@@ -863,7 +872,7 @@ static int
 install_dump_fba(int fd, struct disk_info* info, uint64_t mem)
 {
 	struct boot_fba_stage0 stage0;
-	disk_blockptr_t stage2_list[8];
+	disk_blockptr_t stage2_list[16];
 	blocknum_t block;
 	blocknum_t count;
 	void* buffer;
@@ -880,7 +889,7 @@ install_dump_fba(int fd, struct disk_info* info, uint64_t mem)
 	if (rc)
 		return rc;
 	count = (size + info->phy_block_size - 1) / info->phy_block_size;
-	if (count > 8) {
+	if (count > 16) {
 		error_reason("FBA dump record is too large");
 		free(buffer);
 		return -1;
@@ -1017,7 +1026,7 @@ install_dump(const char* device, struct job_target_data* target, uint64_t mem)
 	switch (info->type) {
 	case disk_type_eckd_classic:
 	case disk_type_eckd_compatible:
-		if (check_partition_bounds(info)) {
+		if (check_eckd_dump_partition(info)) {
 			error_text("Dump target '%s'", device);
 			rc = -1;
 			break;
@@ -1128,7 +1137,7 @@ install_mvdump(char* const device[], struct job_target_data* target, int count,
 					       info[i]->device, device[i]);
 		if (rc)
 			goto out;
-		if (check_partition_bounds(info[i])) {
+		if (check_eckd_dump_partition(info[i])) {
 			error_text("Dump target '%s'", device[i]);
 			rc = -1;
 			goto out;
